--
-- MPSeller
--
-- @author  Destroileur
-- Version 1.0
-- @date 20.10.2017

MPSeller = {};
MPSeller.modDir = g_currentModDirectory;


function MPSeller:fLog(str)

    print(str);

end;

function MPSeller:loadMap(name)

	self.username = "";
	self.masterUsername = "";
	self.isSeller = false;
	self.otherSeller = false;
	
	self:getUsername();

	self.lastSeller = "";
	self.currentSeller = "";
	self.lastSellerTime = 0;
	self.lastTransactionTime = 0;
	self.lastMoney = {};
	
	self.maxTransactionTime = 30;
	self.remainingTime = 0;
	
    self.is_seller_icon = createImageOverlay(MPSeller.modDir.."is_seller.dds");
	self.other_seller_icon = createImageOverlay(MPSeller.modDir.."other_seller.dds");
	
	self.maxTransactionTime = self.maxTransactionTime;
	if g_currentMission:getIsServer() then
		self.timer = addTimer(1000, "checkTime", self);
	end;
	
	self.checkIsBusy = false

end;


function MPSeller:checkTime()

	self.lastSellerTime = self.lastSellerTime + 1;
	self.lastTransactionTime = self.lastTransactionTime + 1;
	
	if not (self.currentSeller == "") then
	
		local remainingTime = tostring(self.maxTransactionTime - self.lastTransactionTime);
		self:ClientController("%all", "refreshRemainingTime", remainingTime);
		ClientControllerEvent.sendEvent("%all", "refreshRemainingTime", remainingTime);
	
		if not (self.checkIsBusy) then
	
			self.checkIsBusy = true;
		
			for bb=1, table.getn(g_currentMission.users) do
			
				local checkUsername = g_currentMission.users[bb].nickname;
				
				if not (checkUsername == self.currentSeller) then
				
					local sendMoney = 0;
					local currentMoney = nil;
					
					if (self:isMaster(checkUsername)) then
						currentMoney = g_currentMission.missionStats.money;
					else
						currentMoney = self:getWalletMoney(checkUsername);
					end;

					if not (self.lastMoney[checkUsername] == nil) then
						if (currentMoney > self.lastMoney[checkUsername]) then
							self.lastTransactionTime = 0;
							sendMoney = currentMoney - self.lastMoney[checkUsername];
						end;
					end;
					self.lastMoney[checkUsername] = currentMoney;

					if (sendMoney > 0) then
						local newSenderMoney = currentMoney - sendMoney;
						self:setWalletMoney(checkUsername,newSenderMoney);
						local newTargetMoney = self:getWalletMoney(self.currentSeller)+sendMoney;
						self:setWalletMoney(self.currentSeller,newTargetMoney);
						if (self:isMaster(checkUsername)) then
							self.lastMoney[checkUsername] = g_currentMission.missionStats.money;
						else
							self.lastMoney[checkUsername] = newSenderMoney;
						end;
					end;
					
				end;
				
			end;
			
			if (self.lastTransactionTime >= self.maxTransactionTime) then
				self.lastSeller = self.currentSeller;
				self.lastSellerTime = 0;
				self.currentSeller = "";
				self:ClientController("%all", "refreshCurrentSeller", "");
				ClientControllerEvent.sendEvent("%all", "refreshCurrentSeller", "");
			end;
			
			self.checkIsBusy = false;
			
		end;
		
	
	end;
		
	return true;
	
end;


function MPSeller:getWalletMoney(username)

	local WalletMoney = 0;
	
	if self:isMaster(username) then
		WalletMoney = g_currentMission.missionStats.money;
	else
		for a=1, table.getn(g_currentMission.users) do
			if g_currentMission.users[a].nickname == username then
				WalletMoney = g_currentMission.users[a].money;
				break;
			end;
		end;
	end;
	
	
    
    return WalletMoney;
end;

function MPSeller:setWalletMoney(username, money)

	if self:isMaster(username) then
		g_currentMission.missionStats.money = money;
	else
		for a=1, table.getn(g_currentMission.users) do
			if g_currentMission.users[a].nickname == username then
				g_currentMission.users[a].money = money;
				break;
			end;
		end;
	end;
end;

function MPSeller:getUsername()

	if (self.username == "") then
		for a=1, table.getn(g_currentMission.users) do
			local user = g_currentMission.users[a];
			if user.userId == g_currentMission.playerUserId then
				self.username = user.nickname;
				break;
			end;
		end;
	end;
end;

function MPSeller:update(dt)

	if self.isSeller == false and self.otherSeller == false then
		g_currentMission:addHelpButtonText(g_i18n:getText("input_SENDSELLERREQUEST"), InputBinding.SENDSELLERREQUEST, nil, GS_PRIO_HIGH);
	end;
	
	if InputBinding.hasEvent(InputBinding.SENDSELLERREQUEST) then
		if g_currentMission:getIsServer() then
			self:ServerController(self.username, "sellerRequest", "");
		else
			ServerControllerEvent.sendEvent(self.username, "sellerRequest", "");
		end;
	end;
	
end;



function MPSeller:draw()

	local posX = g_currentMission.infoBarBgOverlay.x + g_currentMission.infoBarBgOverlay.width - g_currentMission.moneyTextOffsetX;
    local posY = g_currentMission.infoBarBgOverlay.y;

    if self.isSeller then
		setTextColor(0, 1, 1, 1);
		renderOverlay(self.is_seller_icon, posX, posY, 0.0296, 0.0556);
		renderText(posX, posY, 0.036, tostring(self.remainingTime));
	end;
	
	if self.otherSeller then
		setTextColor(0, 1, 1, 1);
		renderOverlay(self.other_seller_icon, posX, posY, 0.0296, 0.0556);
		renderText(posX, posY, 0.036, tostring(self.remainingTime));
	end;

end;

function MPSeller:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MPSeller:isMaster(username)
	if not (username == "") then
		if g_currentMission:getIsServer() then
			self.masterUsername = self.username;
		end;
		if (username == self.masterUsername) then
			return true;
		end;
		for a=1, table.getn(g_currentMission.users) do
			if g_currentMission.users[a].nickname == username then
				if g_currentMission.users[a].isMasterUser then
					self.masterUsername = username;
					return true;
				end;
			end;
		end;
	end;
    return false;
end;

function MPSeller:SendToClient(actionTarget, actionType, actionData)

	if g_currentMission:getIsServer() then
		if (actionTarget == self.username) then
			self:ClientController(actionTarget, actionType, actionData);
		else
			ClientControllerEvent.sendEvent(actionTarget, actionType, actionData);
		end;
	else
		self:fLog("## MPSeller ## ERROR : Can not send client to client !");
	end;

end;

function MPSeller:SendToServer(actionSender, actionType, actionData)

	if g_currentMission:getIsServer() then
		self:ServerController(actionSender, actionType, actionData);
	else
		ServerControllerEvent.sendEvent(actionSender, actionType, actionData);
	end

end;



function MPSeller:ClientController(actionTarget, actionType, actionData)

	if (actionTarget == self.username or actionTarget == "%all") then
		if actionType == "refreshCurrentSeller" then
			if (actionData == "") then
				if (self.isSeller == true) then
					g_currentMission:showBlinkingWarning(g_i18n:getText("END_SELLER"), 3000);
				end;
				self.isSeller = false;
				self.otherSeller = false;
				self.remainingTime = self.maxTransactionTime;
			else
				if (actionData == self.username) then
					g_currentMission:showBlinkingWarning(g_i18n:getText("NOW_SELLER"), 3000);
					self.isSeller = true;
					self.otherSeller = false;
				else
					self.otherSeller = true;
					self.isSeller = false;
				end;
			end;
		elseif actionType == "refreshRemainingTime" then
			self.remainingTime = tonumber(actionData);
		else
			self:fLog("## MPSeller ## ERROR : Invalid client action '"..actionType.."' from server");
		end;
	end;
	
end;

function MPSeller:ServerController(actionSender, actionType, actionData)

	if actionType == "sellerRequest" then
		local canBeSeller = false;
		if (self.currentSeller == "") then
			if (self.lastSeller == actionSender) then
				if (self.lastSellerTime >= 5) then
					canBeSeller = true;
				end;
			else
				canBeSeller = true;
			end;
		end;
		if (canBeSeller) then
			self.lastMoney = {};
			self.lastTransactionTime = 0;
			self.currentSeller = actionSender;
			self:ClientController("%all", "refreshCurrentSeller", actionSender);
			ClientControllerEvent.sendEvent("%all", "refreshCurrentSeller", actionSender);
		end;
    else
		self:fLog("## MPSeller ## ERROR : Invalid action server '"..actionType.."' from "..actionSender);
    end;

	
end;


function MPSeller:keyEvent(unicode, sym, modifier, isDown)
	self:getUsername();
end;

addModEventListener(MPSeller);

function MPSeller:deleteMap()
end;